#!/usr/bin/env python

# test_triggerdelay.py
# Copyright (C) 2013  Bastian Venthur
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


from __future__ import division

import unittest
import json
import socket
import time
import math

import libmushu
from libmushu.driver.randomamp import RandomAmp
from libmushu.amplifier import Amplifier
import logging

import numpy as np
from matplotlib import pyplot as plt


logging.basicConfig(format='%(relativeCreated)10.0f %(processName)-11s %(threadName)-10s %(name)-10s %(levelname)8s %(message)s', level=logging.NOTSET)
logger = logging.getLogger(__name__)
logger.info('Logger started')


class TriggerTestAmp(Amplifier):
    """TriggerTestAmp.

    This amp sends marker just before and after its blocking sleep, along with
    the current timestamp as payload. On the receiving side one can use the
    payload to calculate the delay between sending and receiving triggers via
    TCP/IP.
    """

    def __init__(self):
        self.channels = 100
        self.fs = 100
        self.last_sample = time.time()

    def start(self):
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.connect(('localhost', 12345))

    @property
    def sample_len(self):
        return 1. / self.fs

    @property
    def elapsed(self):
        return time.time() - self.last_sample

    def get_data(self):
        # simulate blocking until we have enough data
        elapsed = self.elapsed
        if elapsed < self.sample_len:
            time.sleep(self.sample_len - elapsed)
        self.s.sendall("%f\n" % time.time())
        dt = self.elapsed
        samples = math.floor(self.fs * dt)
        data = np.random.randint(0, 1024, (samples, self.channels))
        self.last_sample = time.time()
        return data, []

    def configure(self, fs):
        self.fs = fs

    def get_sampling_frequency(self):
        return self.fs


class TestTriggerDelay(unittest.TestCase):
    """Test the trigger delay."""

    def test_triggerdelay(self):
        """Mean and max delay must be reasonably small."""
        for i in 10, 100, 1000, 10000:
            print (1000 / i)
            logger.debug('Setting FS to {fs}kHz'.format(fs=(i / 1000)))
            amp = libmushu.AmpDecorator(TriggerTestAmp)
            amp.configure(fs=i)
            amp.start()
            markers_second = []
            t_start = time.time()
            delays = []
            while time.time() < t_start + 1:
                data, marker = amp.get_data()
                t = time.time()
                print marker
                print len(data)
                for idx, m in enumerate(marker):
                    delays.append(float(m[0]) + (len(marker) - idx) * (1000 / i))
                markers_second.extend([x for x, y in marker])
            amp.stop()
            markers_second = np.array(markers_second)
            delays = np.array(delays)
            delays *= 1000
            logger.debug("Min: %.2f, Max: %.2f, Mean: %.2f, Std: %.2f" % (markers_second.min(), markers_second.max(), markers_second.mean(), markers_second.std()))
            logger.debug("Min: %.2f, Max: %.2f, Mean: %.2f, Std: %.2f" % (delays.min(), delays.max(), delays.mean(), delays.std()))
            logger.debug('')

            #self.assertLessEqual(delays.mean(), 1)
            #self.assertLessEqual(delays.max(), 10)

if __name__ == '__main__':
    unittest.main()



#!/usb/bin/env python

# gtec.py
# Copyright (C) 2013  Bastian Venthur
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


# TODO: update to new version of pyusb

import struct
import time
from exceptions import Exception
import logging

import usb
from scipy.signal import iirfilter
import numpy as np

from libmushu.amplifier import Amplifier


logger = logging.getLogger(__name__)
logger.info('Logger started')

ID_VENDOR_GTEC = 0x153c
# I saw an am with this vendorid too
ID_VENDOR_GTEC2 = 0x15c3
ID_PRODUCT_GUSB_AMP = 0x0001

CX_OUT = usb.TYPE_VENDOR | usb.ENDPOINT_OUT


class GUSBamp(Amplifier):

    def __init__(self):
        logger.info('Initializing GUSBamp instance')
        # list of available amps
        self.amps = []
        for bus in usb.busses():
            for device in bus.devices:
                if (device.idVendor in [ID_VENDOR_GTEC, ID_VENDOR_GTEC2] and
                    device.idProduct == ID_PRODUCT_GUSB_AMP):
                    self.amps.append(device)
        self.devh = None
        self.mode = None
        # Initialize the amplifier and make it ready.
        device = self.amps[0]
        self.devh = device.open()
        # detach kernel driver if nessecairy
        config = device.configurations[0]
        self.devh.setConfiguration(config)
        assert(len(config.interfaces) > 0)
        # sometimes it is the other one
        first_interface = config.interfaces[0][0]
        if first_interface is None:
            first_interface = config.interfaces[0][1]
        first_setting = first_interface.alternateSetting
        self.devh.claimInterface(first_interface)
        self.devh.setAltInterface(first_interface)
        # initialization straight from the usb-dump
        self.set_mode('data')
        self.devh.controlMsg(CX_OUT, 0xb6, value=0x80, buffer=0)
        self.devh.controlMsg(CX_OUT, 0xb5, value=0x80, buffer=0)
        self.devh.controlMsg(CX_OUT, 0xb9, value=0x00, buffer="\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10")
        self.set_slave_mode(False)
        self.devh.controlMsg(CX_OUT, 0xd3, value=0x01, buffer=0)
        self.devh.controlMsg(CX_OUT, 0xca, value=0x01, buffer=0)
        self.devh.controlMsg(CX_OUT, 0xc8, value=0x01, buffer="\x00"*16)
        self.set_common_reference()
        self.set_common_ground()
        self.set_calibration_mode('sine')
        self.set_sampling_ferquency(128, [False for i in range(16)], None, None)

    def start(self):
        self.devh.controlMsg(CX_OUT, 0xb5, value=0x08, buffer=0)
        self.devh.controlMsg(CX_OUT, 0xf7, value=0x00, buffer=0)

    def stop(self):
        self.devh.controlMsg(CX_OUT, 0xb8, [])

    def get_data(self):
        """Get data."""
        # TODO: should we use numpy arrays right here?
        # TODO: what is the in-endpoint
        # 0x2 or 0x86
        endpoint = 0x86
        # TODO what is the optimal number here
        size = 2028 #512
        try:
            # TODO what is the optimal timeout here?
            data = self.devh.bulkRead(endpoint, size, 100)
        except usb.USBError:
            data = []
        data = ''.join(map(chr, data))
        data = np.fromstring(data, np.float32, len(data)/4)
        try:
            data = data.reshape(-1, 17)
        except:
            logger.error("Got incomplete packet from the amp, discarding it!")
            data = np.array([]).reshape(-1, 17)
        if self.mode == 'impedance':
            data = self.calculate_impedance(data)
        elif self.mode == 'data':
            # get data in mV
            data /= 8.15
        return data, []

    def get_channels(self):
        return [str(i) for i in range(17)]

    @staticmethod
    def is_available():
        for bus in usb.busses():
            for device in bus.devices:
                if (device.idVendor in [ID_VENDOR_GTEC, ID_VENDOR_GTEC2] and
                    device.idProduct == ID_PRODUCT_GUSB_AMP):
                    return True
        return False

    ###########################################################################
    # Low level amplifier methods
    ###########################################################################

    def set_mode(self, mode):
        """Set mode, 'impedance', 'data'."""
        if mode == 'impedance':
            self.devh.controlMsg(CX_OUT, 0xc9, value=0x00, buffer=0)
            self.devh.controlMsg(CX_OUT, 0xc2, value=0x03, buffer=0)
            self.mode = 'impedance'
        elif mode == 'calibrate':
            self.devh.controlMsg(CX_OUT, 0xc1, value=0x00, buffer=0)
            self.devh.controlMsg(CX_OUT, 0xc2, value=0x02, buffer=0)
            self.mode = 'calibration'
        elif mode == 'data':
            self.devh.controlMsg(CX_OUT, 0xc0, value=0x00, buffer=0)
            self.devh.controlMsg(CX_OUT, 0xc2, value=0x01, buffer=0)
            self.mode = 'data'
        else:
            raise AmpError('Unknown mode: %s' % mode)


    def set_sampling_ferquency(self, fs, channels, bpfilter, notchfilter):
        """ Set the sampling frequency and filters for individual channels.

        Parameters:
        fs -- sampling frequency
        channels -- list of booleans: channels[0] == True: enable filter for channel 0
        bpfilter -- tuple: parameters for the band pass filter (hp, lp, fs, order) or None
        notchfilter -- tuple: parameters for the band stop filter (hp, lp, fs, order) or None

        """
        # we have: hp, lp, fs, order, typ
        # signal.iirfilter(order/2, [hp/(fs/2), lp/(fs/2)], ftype='butter', btype='band')
        # we get 18 coeffs and put them in as '<d' in the buffer
        # struct.pack('<'+'d'*18, *coeffs)

        # special filter: means no filter
        null_filter = "\x00\x00\x00\x00\x00\x00\xf0\x3f"+"\x00\x00\x00\x00\x00\x00\x00\x00"*17

        if bpfilter:
            bp_hp, bp_lp, bp_fs, bp_order = bpfilter
            bp_b, bp_a = iirfilter(bp_order/2, [bp_hp/(bp_fs/2), bp_lp/(bp_fs/2)], ftype='butter', btype='band')
            bp_filter = list(bp_b)
            bp_filter.extend(list(bp_a))
            bp_filter = struct.pack("<"+"d"*18, *bp_filter)
        else:
            bp_filter = null_filter

        if notchfilter:
            bs_hp, bs_lp, bs_fs, bs_order = notchfilter
            bs_b, bs_a = iirfilter(bs_order/2, [bs_hp/(bs_fs/2), bs_lp/(bs_fs/2)], ftype='butter', btype='bandstop')
            bs_filter = list(bs_b)
            # the notch filter has (always?) an order of 4 so fill the gaps with
            # zeros
            if len(bs_filter) < 9:
                diff = 9 - len(bs_filter)
                bs_filter.extend([0.0 for i in range(diff)])
            bs_filter.extend(list(bs_a))
            if len(bs_filter) < 18:
                diff = 18 - len(bs_filter)
                bs_filter.extend([0.0 for i in range(diff)])
            bs_filter = struct.pack("<"+"d"*18, *bs_filter)
        else:
            bs_filter = null_filter

        # set the filters for all channels
        if bpfilter == notchfilter == None:
            self.devh.controlMsg(CX_OUT, 0xc6, value=0x01, buffer=bp_filter)
            self.devh.controlMsg(CX_OUT, 0xc7, value=0x01, buffer=bs_filter)
        else:
            idx = 1
            for i in channels:
                if i:
                    self.devh.controlMsg(CX_OUT, 0xc6, value=idx, buffer=bp_filter)
                    self.devh.controlMsg(CX_OUT, 0xc7, value=idx, buffer=bs_filter)
                idx += 1

        # set the sampling frequency
        self.devh.controlMsg(CX_OUT, 0xb6, value=fs, buffer=0)


    def set_calibration_mode(self, mode):
        # buffer: [0x03, 0xd0, 0x07, 0x02, 0x00, 0xff, 0x07]
        #          ====  ==========
        # (1) mode:
        # (2) amplitude: little endian (0x07d0 = 2000)
        if mode == 'sine':
            self.devh.controlMsg(CX_OUT, 0xcb, value=0x00, buffer="\x03\xd0\x07\x02\x00\xff\x07")
        elif mode == 'sawtooth':
            self.devh.controlMsg(CX_OUT, 0xcb, value=0x00, buffer="\x02\xd0\x07\x02\x00\xff\x07")
        elif mode == 'whitenoise':
            self.devh.controlMsg(CX_OUT, 0xcb, value=0x00, buffer="\x05\xd0\x07\x02\x00\xff\x07")
        elif mode == 'square':
            self.devh.controlMsg(CX_OUT, 0xcb, value=0x00, buffer="\x01\xd0\x07\x02\x00\xff\x07")
        else:
            raise AmpError('Unknown mode: %s' % mode)

    def calculate_impedance(self, u_measured, u_applied=1e4):
        return (u_measured * 1e6) / (u_applied - u_measured) - 1e4


    def set_common_ground(self, a=False, b=False, c=False, d=False):
        """Set common ground for the electrodes.

        Parameters:
            a, b, c, d -- correspond to the groups on the amp, either of them
                can be true or false

        """
        v = (d << 3) + (c << 2) + (b << 1) + a
        self.devh.controlMsg(CX_OUT, 0xbe, value=v, buffer=0)


    def set_common_reference(self, a=False, b=False, c=False, d=False):
        """Set common reference for the electrodes.

        Parameters:
            a, b, c, d -- correspond to the groups on the amp, either of them
                can be true or false

        """
        v = (d << 3) + (c << 2) + (b << 1) + a
        self.devh.controlMsg(CX_OUT, 0xbf, value=v, buffer=0)


    def set_slave_mode(self, slave):
        """Set amp into slave or master mode.

        Parameters:
            slave -- if true, set into slave mode, set to master otherwise

        """
        v = 1 if slave else 0
        self.devh.controlMsg(CX_OUT, 0xcd, value=v, buffer=0)


class AmpError(Exception):
    pass




def main():
    amp = GUSBamp()
    amp.start()
    try:
        while True:
            t = time.time()
            data = amp.get_data()
            dt = time.time() - t
            if len(data) > 0:
                print "%.5f seconds (%.5f ps), length: %d" % (dt, (len(data) / 16.) * 1/dt, len(data))
    finally:
        amp.stop()

if __name__ == '__main__':
    import sys
    import cProfile
    if len(sys.argv) > 1 and sys.argv[1].startswith('prof'):
        cProfile.run('main()', 'prof')
    else:
        main()




#!/usr/bin/env python

# mushu.py
# Copyright (C) 2013  Bastian Venthur
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


import logging
import time
import ttk
import Tkinter as tk

import matplotlib
matplotlib.use('TkAgg')

from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as FigureCanvas

import numpy as np

import libmushu

logging.basicConfig(format='%(relativeCreated)10.0f %(threadName)-10s %(name)-10s %(levelname)8s %(message)s', level=logging.NOTSET)
logger = logging.getLogger(__name__)
logger.info('Logger started')


class Gui(ttk.Frame):

    def __init__(self, master):
        self.amp_started = False

        ttk.Frame.__init__(self, master)
        self.master.title('Mushu')
        self.pack()

        self.available_amps = libmushu.get_available_amps()

        frame = tk.Frame(self)
        frame.pack(fill=tk.BOTH, expand=1)

        self.label1 = ttk.Label(frame, text='Select Amplifier')
        self.label1.grid(column=0, row=0, sticky='we')
        self.amp_combobox = ttk.Combobox(frame, values=[str(i) for i in self.available_amps])
        self.amp_combobox.grid(column=0, row=1, sticky='we')
        self.amp_combobox.bind("<<ComboboxSelected>>", self.on_amplifier_selected)

        self.label2 = ttk.Label(frame, text='Select Configuration Preset')
        self.label2.grid(column=1, row=0, sticky='we')
        self.config_combobox = ttk.Combobox(frame)
        self.config_combobox.grid(column=1, row=1, sticky='we')
        self.config_combobox.bind("<<ComboboxSelected>>", self.onComboboxSelected)

        self.label3 = ttk.Label(frame, text='Start/Stop Amplifier')
        self.label3.grid(column=2, row=0, sticky='we')
        self.start_stop_button = ttk.Button(frame, text='Start', command=self.onStartStopButtonClicked)
        self.start_stop_button.grid(column=2, row=1, sticky='we')

        # set up the figure
        fig = Figure()
        self.canvas = FigureCanvas(fig, master=self.master)
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)
        self.canvas.show()
        self.axis = fig.add_subplot(111)

        self.PAST_POINTS = 256
        self.SCALE = 30000

        self.channels = []
        self.n_channels = 0

        self.init_plot()
        self.master.after_idle(self.visualizer)

    def onStartStopButtonClicked(self):
        logger.debug('Start.')
        if self.amp_started:
            logger.debug('Stop.')
            self.amp.stop()
            self.start_stop_button.config(text='Start')
            self.amp_started = False
        else:
            logger.debug('Start.')
            self.amp.start()
            self.start_stop_button.config(text='Stop')
            self.amp_started = True

    def onComboboxSelected(self, event):
        idx = event.widget.current()
        cfg = self.amp.presets[idx][1]
        self.amp.configure(**cfg)

    def init_plot(self):
        self.axis.lines = []
        for i in range(self.n_channels):
            self.axis.plot(0)
        self.canvas.draw()
        if self.n_channels == 0:
            self.data = np.array([]).reshape(-1, 1)
        else:
            self.data = np.array([]).reshape(-1, self.n_channels)
        self.data_buffer = []
        self.t2 = time.time()
        self.k = 0
        self.nsamples = 0

    def on_amplifier_selected(self, event):
        idx = event.widget.current()
        ampstr = self.available_amps[idx]
        amp = libmushu.get_amp(ampstr)
        self.set_amplifier(amp)

    def set_amplifier(self, amp):
        self.config_combobox.configure(values = [txt for txt, _ in amp.presets])
        self.amp = amp
        self.channels = amp.get_channels()
        self.n_channels = len(self.channels)

    def visualizer(self):
        if self.amp_started:
            tmp, marker = self.amp.get_data()
            # display #samples / second
            if tmp is not None:
                self.nsamples += tmp.shape[0]
                self.k += 1
                if self.k == 100:
                    sps = self.nsamples / (time.time() - self.t2)
                    logger.debug('%.2f samples / second\r' % sps)
                    self.t2 = time.time()
                    self.nsamples = 0
                    self.k = 0
            # check if nr of channels has changed since the last probe
            if tmp.shape[1] != self.data.shape[1]:
                logger.debug('Number of channels has changed, re-initializing the plot.')
                self.channels = self.amp.get_channels()
                self.n_channels = len(self.channels)
                self.init_plot()
            # append the new data
            new_data = tmp
            self.data = np.concatenate([self.data, new_data])
            self.data = self.data[-self.PAST_POINTS:]
            # plot the data
            data_clean = self.normalize(self.data)
            dmin = data_clean.min()
            dmax = data_clean.max()
            dr = (dmax - dmin) * 0.7
            SCALE = dr
            ticklocs = []
            x = [i for i in range(len(self.data))]
            for j, line in enumerate(self.axis.lines):
                line.set_xdata(x)
                #line.set_ydata(self.data[:, j] + j * SCALE)
                line.set_ydata(data_clean[:, j] + j * SCALE)
                ticklocs.append(j * SCALE)
            self.axis.set_ylim(-SCALE, self.n_channels * SCALE)
            self.axis.set_xlim(i - self.PAST_POINTS, i)
            self.axis.set_yticks(ticklocs)
            self.axis.set_yticklabels(self.channels)
            self.canvas.draw()
            #logger.debug('%.2f FPS' % (1 / (time.time() - t)))
        self.master.after(10, self.visualizer)

    def normalize(self, data):
        return data - np.average(data)

if __name__ == '__main__':
    root = tk.Tk()
    gui = Gui(root)
    gui.mainloop()




# amplifier.py
# Copyright (C) 2013  Bastian Venthur
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


"""
This module provides the :class:`Amplifier` class, which is the base
class of all low level amplifier drivers. If you want to write a driver
for a specific amplifier your driver must derive from the
:class:`Amplifier` class and implement its methods.

Users will not use your driver directly but a decorated version of it
which provides additional features like writing data to a file and
receiving marker via TCP.

"""

class Amplifier(object):
    """Amplifier base class.

    The base class is very generic on purpose. Amplifiers from different
    vendors vary in so many ways that it is difficult to find a common
    set of methods that all support.

    In the spirit of "encapsulating what varies", I decided to
    encapsulate the configuration. So the main configuration of the
    amplifier, like setting the mode (e.g. data, impedance, etc.),
    sampling frequency, etc. happens in :meth:`configure` and is very
    specific for the amplifier at hand.

    :meth:`start`, :meth:`stop`, and :meth:`get_data` is very generic
    and must be supported by all derived amplifier classes.

    How an amplifier should be used::

        amp = Amp()

        # measure impedance
        amp.configure(**config)
        amp.start()
        while 1:
            data = amp.get_data()
            if enough:
                break
        amp.stop()

        # measure data
        amp.configure(**config)
        channels = amp.get_channels()
        amp.start()
        while 1:
            data = amp.get_data()
            if enough:
                break
        amp.stop()

    """

    presets = []


    def configure(self, **kwargs):
        """Configure the amplifier.

        Use this method to set the mode (i.e. impedance, data, ...),
        sampling frequency, filter, etc.

        This depends strongly on the amplifier.

        Parameters
        ----------
        kwargs : dict
            the configuration of the amplifier

        """
        pass

    def start(self):
        """Make the amplifier ready for delivering data."""
        pass

    def stop(self):
        """Stop the amplifier."""
        pass

    def get_data(self):
        """Get data from the amplifier.

        This method is called as fast as possible (e.g. hundreds of
        times per second) and returns the data and the marker (if
        supported).

        Returns
        -------
        data : ndarray
            a numpy array (time, channels) of the EEG data
        markers : list of (float, str)
            a list of markers. Each element is a tuple of timestamp and
            string. The timestamp is the time in ms relative to the
            onset of the block of data. Note that negative values are
            *allowed* as well as values bigger than the length of the
            block of data returned. That is to be interpreted as a
            marker from the last block and a marker for a future block
            respectively.

        Examples
        --------

        Create a very slow amplifier with 1Hz and 3 channels and send
        some markers:

        >>> amp = libmushu.get_amp('randomamp')
        >>> amp.configure(fs=1, channels=3)
        >>> amp.start()
        >>> while True:
        ...     time.sleep(.5)
        ...     data, marker = amp.get_data()
        ...     print '---'
        ...     print data
        ...     print marker
        ...
        ---
        [[590 938  72]]
        [[98.75297546386719, 'foo'], [553.4558296203613, 'bar']]
        ---
        [[167 168  40]]
        []
        ---
        [[727 705 934]]
        [[16.066789627075195, 'baz']]

        """
        pass

    def get_channels(self):
        """Return the list of channel names.

        The list has the same order as the data, i.e. the second name in
        the list represents the second colum of the data returned by
        :meth:`get_data`.

        Returns
        -------
        channels : list of strings
            the channel names

        """
        raise NotImplementedError

    def get_sampling_frequency(self):
        """Return the sampling frequency.

        This method returns the sampling frequency which is currently
        enabled in the amplifier.

        Returns
        -------
        fs : float
            the sampling frequency

        """
        raise NotImplementedError

    @staticmethod
    def is_available():
        """Is the amplifier connected to the computer?

        This method should be overwritten by derived classes and return
        True if the amplifier is connected to the computer or False if
        not.

        Returns
        -------
        available : boolean

        """
        raise NotImplementedError




from __future__ import division

import time
import logging

import numpy as np
import pylsl

from libmushu.amplifier import Amplifier


logger = logging.getLogger(__name__)
logger.info('Logger started.')


class LSLAmp(Amplifier):
    """Pseudo Amplifier for lab streaming layer (lsl).

    This amplifier connects to an arbitrary EEG device that is
    publishing its data via lsl to the network. With this class you can
    use this amplifier like a normal `mushu` amplifier.

    https://code.google.com/p/labstreaminglayer/

    Examples
    --------

    >>> amp = libmushu.get_amp('lslamp')
    >>> amp.configure()
    >>> amp.start()
    >>> while True:
    ...     data, marker = amp.get_data()
    ...     # do something with data and/or break the loop
    >>> amp.stop()

    """

    def configure(self):
        """Configure the lsl device.

        This method looks for open lsl streams and picks the first `EEG`
        and `Markers` streams and opens lsl inlets for them.

        Note that lsl amplifiers cannot be configured via lsl, as the
        protocol was not designed for that. You can only connect (i.e.
        subscribe) to devices that connected (publishing) via the lsl
        protocol.

        """
        # lsl defined
        self.max_samples = 1024
        # open EEG stream
        streams = pylsl.resolve_stream('type', 'EEG')
        if len(streams) > 1:
            logger.warning('Number of EEG streams is > 0, picking the first one.')
        self.lsl_inlet = pylsl.StreamInlet(streams[0])
        # open marker stream
        streams = pylsl.resolve_stream('type', 'Markers')
        if len(streams) > 1:
            logger.warning('Number of Marker streams is > 0, picking the first one.')
        self.lsl_marker_inlet = pylsl.StreamInlet(streams[0])
        info = self.lsl_inlet.info()
        self.n_channels = info.channel_count()
        self.fs = info.nominal_srate()
        # storage for markers that arrived w/o samples
        self._markers = []
        self._m_timestamps = []

    def start(self):
        """Open the lsl inlets.

        """
        logger.debug('Opening lsl streams.')
        self.lsl_inlet.open_stream()
        self.lsl_marker_inlet.open_stream()

    def stop(self):
        """Close the lsl inlets.

        """
        logger.debug('Closing lsl streams.')
        self.lsl_inlet.close_stream()
        self.lsl_marker_inlet.close_stream()

    def get_data(self):
        """Receive a chunk of data an markers.

        Returns
        -------
        chunk, markers: Markers is time in ms since relative to the
        first sample of that block.

        """
        samples, timestamps = self.lsl_inlet.pull_chunk(timeout=0.0, max_samples=self.max_samples)
        markers, m_timestamps = self.lsl_marker_inlet.pull_chunk(timeout=0.0, max_samples=self.max_samples)

        # put any leftover markers back into the loop
        if self._markers:
            markers = self._markers + markers
            m_timestamps = self._m_timestamps + m_timestamps
            self._markers = []
            self._m_timestamps = []

        samples = np.array(samples).reshape(-1, self.n_channels)
        markers = np.array(markers)

        m_timestamps = np.array(timestamps)
        if len(m_timestamps) > 0:
            if len(timestamps) > 0:
                m_timestamps -= timestamps[0]
            else:
                # we received markers, but no data, so we cannot
                # calculate the relative time to the first sample, we
                # store the markers until the next sample arrived
                self._markers.extend(markers)
                self._m_timestamps.extend(m_timestamps)
                markers = []
                m_timestamps = []

        # convert timestamps to ms
        m_timestamps *= 1000

        return samples, zip(markers, m_timestamps)

    def get_channels(self):
        """Get channel names.

        """
        ['Ch %i' % i for i in range(self.n_channels)]
        return self.channels

    def get_sampling_frequency(self):
        """Get the sampling frequency of the lsl device.

        """
        return self.fs

    @staticmethod
    def is_available():
        """Check if an lsl stream is available on the network.

        Returns
        -------

        ok : Boolean
            True if there is a lsl stream, False otherwise

        """
        # Return True only if at least one lsl stream can be found on
        # the network
        if pylsl.resolve_streams():
            return True
        return False




# ampdecorator.py
# Copyright (C) 2013  Bastian Venthur
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


"""
This module provides the :class:`AmpDecorator` class.

As a user, it is very unlikely that you'll have to deal with it
directly. Its main purpose is to add additional functionality to the low
level amplifier drivers. This functionality includes features like:
saving data to a file. or being able to receive marker via TCP/IP.

By using the :func:`libmushu.__init__.get_amp` method, you'll
automatically receive decorated amplifiers.

"""

from __future__ import division

import select
import socket
import time
from multiprocessing import Process, Queue, Event
import os
import struct
import json
import logging

from libmushu.amplifier import Amplifier


logger = logging.getLogger(__name__)
logger.info('Logger started')


END_MARKER = '\n'
BUFSIZE = 2**16
PORT = 12345


class AmpDecorator(Amplifier):
    """This class 'decorates' the Low-Level Amplifier classes with
    TCP-Marker and Save-To-File functionality.

    You use it by decorating (not as in Python-Decorator, but in the GoF
    sense) the low level amplifier class you want to use::

        import libmushu
        from libmushu.ampdecorator import AmpDecorator
        from libmushu.driver.randomamp import RandomAmp

        amp = Ampdecorator(RandomAmp)

    Waring: The TCP marker timings on Windows have a resolution of
    10ms-15ms.  On Linux the resolution is 1us. This is due to
    limitations of Python's time.time method, or rather a Windows
    specific issue.

    There exists currently no precise timer, providing times which are
    comparable between two running processes on Windows. The performance
    counter provided on Windows, has a much better resolution but is
    relative to the processes start time and it drifts (1s per 100s), so
    it is only precise for a relatively short amount of time.

    If a higher precision is needed one has to replace the time.time
    calls with something which provides a better precision. For example
    one could create a third process which provides times or regularly
    synchronize both processes with the clock synchronization algorithm
    as described here:

        http://en.wikipedia.org/wiki/Network_Time_Protocol

    Alternatively one could use `timeGetTime` from Windows' Multi Media
    library, which is tunable via `timeBeginPeriod` and provides a
    precision of 1-2ms. Apparently this is the way Chrome and many
    others do it.::

        from __future__ import division

        from ctypes import windll
        import time

        timeBeginPeriod = windll.winmm.timeBeginPeriod
        timeEndPeriod = windll.winmm.timeEndPeriod
        timeGetTime = windll.winmm.timeGetTime

        if __name__ == '__main__':
            # wrap the code that needs high precision in timeBegin- and
            # timeEndPeriod with the same parameter. The parameter is
            # the interval in ms you want as precision. Usually the
            # minimum value allowed is 1 (best).
            timeBeginPeriod(1)
            times = []
            t_start = time.time()
            while time.time() < (time.time() + 1):
                times.append(timeGetTime())
            times = sorted(list(set(times)))
            print(1000 / len(times))
            timeEndPeriod(1)

    """

    def __init__(self, ampcls):
        self.amp = ampcls()
        self.write_to_file = False

    @property
    def presets(self):
        return self.amp.presets

    def start(self, filename=None):
        # prepare files for writing
        self.write_to_file = False
        if filename is not None:
            self.write_to_file = True
            filename_marker = filename + '.marker'
            filename_eeg = filename + '.eeg'
            filename_meta = filename + '.meta'
            for filename in filename_marker, filename_eeg, filename_meta:
                if os.path.exists(filename):
                    logger.error('A file "%s" already exists, aborting.' % filename)
                    raise Exception
            self.fh_eeg = open(filename_eeg, 'w')
            self.fh_marker = open(filename_marker, 'w')
            self.fh_meta = open(filename_meta, 'w')
            # write meta data
            meta = {'Channels': self.amp.get_channels(),
                    'Sampling Frequency': self.amp.get_sampling_frequency(),
                    'Amp': str(self.amp)
                    }
            json.dump(meta, self.fh_meta, indent=4)

        # start the marker server
        self.marker_queue = Queue()
        self.tcp_reader_running = Event()
        self.tcp_reader_running.set()
        tcp_reader_ready = Event()
        self.tcp_reader = Process(target=tcp_reader,
                                  args=(self.marker_queue,
                                        self.tcp_reader_running,
                                        tcp_reader_ready)
                                  )
        self.tcp_reader.start()
        logger.debug('Waiting for TCP reader to become ready...')
        tcp_reader_ready.wait()
        logger.debug('TCP reader is ready...')
        # zero the sample counter
        self.received_samples = 0
        # start the amp
        self.amp.start()

    def stop(self):
        # stop the amp
        self.amp.stop()
        # stop the marker server
        self.tcp_reader_running.clear()
        logger.debug('Waiting for TCP reader process to stop...')
        self.tcp_reader.join()
        logger.debug('TCP reader process stopped.')
        # close the files
        if self.write_to_file:
            logger.debug('Closing files.')
            for fh in self.fh_eeg, self.fh_marker, self.fh_meta:
                fh.close()

    def configure(self, **kwargs):
        self.amp.configure(**kwargs)

    def get_data(self):
        """Get data from the amplifier.

        This method is supposed to get called as fast as possible (i.e
        hundreds of times per seconds) and returns the data and the
        markers.

        Returns
        -------
        data : 2darray
            a numpy array (time, channels) of the EEG data
        markers : list of (float, str)
            a list of markers. Each element is a tuple of timestamp and
            string. The timestamp is the time in ms relative to the
            onset of the block of data. Note that negative values are
            *allowed* as well as values bigger than the length of the
            block of data returned. That is to be interpreted as a
            marker from the last block and a marker for a future block
            respectively.

        """
        # get data and marker from underlying amp
        data, marker = self.amp.get_data()

        t = time.time()
        # length in sec of the new block according to #samples and fs
        block_duration = len(data) / self.amp.get_sampling_frequency()
        # abs time of start of the block
        t0 = t - block_duration
        # duration of all blocks except the current one
        duration = self.received_samples / self.amp.get_sampling_frequency()

        # merge markers
        tcp_marker = []
        while not self.marker_queue.empty():
            m = self.marker_queue.get()
            m[0] = (m[0] - t0) * 1000
            tcp_marker.append(m)
        marker = sorted(marker + tcp_marker)
        # save data to files
        if self.write_to_file:
            for m in marker:
                self.fh_marker.write("%f %s\n" % (1000 * duration + m[0], m[1]))
            for t in data:
                for c in t:
                    self.fh_eeg.write(struct.pack("f", c))
        self.received_samples += len(data)
        return data, marker

    def get_channels(self):
        return self.amp.get_channels()

    def get_sampling_frequency(self):
        return self.amp.get_sampling_frequency()


def tcp_reader(queue, running, ready):
    """Marker-reading process.

    This method runs in a seperate process and receives TCP markers.
    Whenever a marker is received, it is put together with a timestamp
    into a queue.

    Parameters
    ----------
    queue : Queue
        this queue is used to send markers to a different process
    running : Event
        this event is used to signal this process to terminate its main
        loop
    ready : Event
        this signal is used to signal the "parent"-process that this
        process is ready to receive marker

    """
    # setup the server socket
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    server_socket.setblocking(0)
    server_socket.bind(('localhost', PORT))
    server_socket.listen(5)
    ready.set()
    # read the socket until 'running' is set to false
    rlist, wlist, elist = [server_socket], [], []
    while running.is_set():
        readables, _, _ = select.select(rlist, wlist, elist, 1)
        t = time.time()
        for sock in readables:
            if sock == server_socket:
                # a new connection is opened
                connection, address = sock.accept()
                logger.debug('Incoming connection from %s', address)
                rlist.append(connection)
                data_buffer = ''
            else:
                data = sock.recv(BUFSIZE)
                if not data:
                    # an open connection was closed
                    logger.debug('Connection closed')
                    sock.close()
                    rlist.remove(sock)
                else:
                    # received maybe incomplete data from a
                    # connection
                    # TODO: are we? I thought we're using TCP?
                    data_buffer = ''.join([data_buffer, data])
                    split = data_buffer.rsplit(END_MARKER, 1)
                    messages = []
                    if len(split) > 1:
                        # data_buffer contains at least one complete
                        # message
                        data_buffer = split[1]
                        messages = split[0].split(END_MARKER)
                        for m in messages:
                            queue.put([t, m])
    logger.debug('Terminated select-loop')
    # close all connections and the socket
    for s in rlist:
        s.close()





from __future__ import division

import time

from libmushu.amplifier import Amplifier


class ReplayAmp(Amplifier):

    def configure(self, data, marker, channels, fs):
        self.data = data
        self.marker = marker
        self.channels = channels
        self.fs = fs

    def start(self):
        self.last_sample_time = time.time()

    def stop(self):
        pass

    def get_data(self):
        """

        Returns
        -------
        chunk, markers: Markers is time in ms since relative to the
        first sample of that block.

        """
        elapsed = time.time() - self.last_sample_time
        #self.last_sample_time = time.time()
        samples = int(self.fs * elapsed)
        elapsed = samples / self.fs
        self.last_sample_time += elapsed
        # data
        chunk = self.data[:samples]
        self.data = self.data[samples:]
        # markers
        markers = filter(lambda x: x[0] < elapsed * 1000, self.marker)
        self.marker = filter(lambda x: x[0] >= elapsed * 1000, self.marker)
        self.marker = map(lambda x: [x[0] - elapsed * 1000, x[1]], self.marker)
        return chunk, markers

    def get_channels(self):
        return self.channels

    def get_sampling_frequency(self):
        return self.fs

    @staticmethod
    def is_available():
        return True



# __init__.py
# Copyright (C) 2013  Bastian Venthur
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


"""
This package contains the low-level drivers for various amplifiers.

As a user, you'll probably not deal with them directly but with their decorated
counterparts via :func:`libmushu.__init__.get_amp`.

If you want to use the low level drivers directly you can use it like this::

    from libmushu.driver.randomamp import RandomAmp
    amp = RandomAmp()

TODO: Add a 'writing your own drivers' section or tutorial.

"""



# test_triggerprecision.py
# Copyright (C) 2013  Bastian Venthur
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


from __future__ import division
import time
import unittest


class TestTimerPrecision(unittest.TestCase):

    def test_precision(self):
        """The precision of the timer must be at least 10us."""
        # collect as many timestamps as possible in 1 second, remove the
        # duplicates and count the number of elements
        times = []
        t_start = time.time()
        while time.time() < t_start + 1:
            times.append(time.time())
        # remove duplicates
        times = list(set(times))
        resolution = (max(times) - min(times)) / len(times)
        self.assertLessEqual(resolution, 10*1e-6)


if __name__ == '__main__':
    unittest.main()



# __init__.py
# Copyright (C) 2013  Bastian Venthur
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


"""
This module provides two functions: :func:`get_available_amps` which
will tell you which amplifiers are currently available on your computer,
and :func:`get_amp` which you can use to get an amplifier instance to
work with.

How to use libmushu with the decorated drivers (recommended)::

    import libmushu

    # you know what amp you want to use:
    amp = libmushu.get_amp('epoc')
    ...

    # Or: you select one of the available amps:
    amps = libmushu.get_available_amps()
    amp = libmushu.get_amp(amps[0])
    ...

How to use the libmushu's low level driver drivers::

    from libmushu.driver.randomamp import RandomAmp
    amp = RandomAmp()

You'll will most likely want to use the decorated drivers and only deal
with the low level drivers if you're a developer or find that the
:class:`libmushu.ampdecorator.AmpDecorator` does not provide the
features you need.

"""


from __future__ import division

from importlib import import_module
import logging

from libmushu.ampdecorator import AmpDecorator

__version__ = '0.2'

__all__ = ['supported_amps', 'get_available_amps', 'get_amp']


# TODO: low level driver must have a real name?
supported_amps = {
    'epoc': ['emotiv', 'Epoc'],
    'gusbamp': ['gtec', 'GUSBamp'],
    'randomamp': ['randomamp', 'RandomAmp'],
    'sinusamp' : ['sinusamp', 'SinusAmp'],
    'replayamp' : ['replayamp', 'ReplayAmp'],
    'lslamp' : ['labstreaminglayer', 'LSLAmp']
}


logger = logging.getLogger(__name__)
logger.info('Logger started')


def get_available_amps():
    """Retrieves all available (e.g. connected) amplifiers.

    This method tests all supported amplifiers if they are connected to
    the system. More precisely: if the amplifiers `is_available` method
    returns True.

    Returns
    -------
    available_amps : list of strings
        a list of the names of the amplifiers which are available

    Examples
    --------

    >>> import libmushu as lm
    >>> lm.get_available_amps()
    ['gusbamp', 'randomamp']

    """
    available_amps = []
    for name, (mod, cls) in supported_amps.items():
        try:
            m = import_module('libmushu.driver.' + mod)
        except ImportError:
            logger.warning('Unable to import %s' % mod, exc_info=True)
            continue
        try:
            c = getattr(m, cls)
            if c.is_available():
                available_amps.append(name)
        except:
            logger.warning('Unable to test if %s is available' % cls, exc_info=True)
    return available_amps


def get_amp(ampname):
    """Get an amplifier instance.


    This factory method takes a low level amplifier driver, wraps it in
    an AmpDecorator and returns an instance.

    Parameters
    ----------
    ampname : str
        the desired amplifier. The string must be a key in the
        :data:`supported_amps` dictionary.

    Returns
    -------
    amp : Amplifier
        an amplifier instance

    Examples
    --------

    >>> import libmushu as lm
    >>> amps = lm.get_available_amps()
    >>> amp = lm.get_amp(amps[0])

    """
    mod, cls = supported_amps.get(ampname)
    m = import_module('libmushu.driver.' + mod)
    c = getattr(m, cls)
    return AmpDecorator(c)





class State(object):
    """Finite-State Machine.

    """

    def __init__(self, name):
        self.name = name


class Input(object):
    pass


class Transition(object):

    def transition(self, input):
        raise NotImplementedError('transition is not implemented yet!')


# define the states:
UNCONFIGURED = 0
CONFIGURED = 1
RUNNING = 2

# define the transitions
CONFIGURE = 0
START = 1
STOP = 2

# define the valdi transitions
# current state: transition -> next state
fsm = {
        UNCONFIGURED : [CONFIGURE, CONFIGURED],
        CONFIGURED : [START, RUNNING],
        RUNNING : [STOP, CONFIGURED]
}

class FSM(object):

    def __init__(self, state_transition_table, current_state):
        self.stt = state_transition_table
        self.curent_state = current_state

    def transition(self, transition):
        allowed_transitions = [t for t, s in self.stt.get(self.current_state, [])]
        if transition in allowed_transitions:
            next_state = [s for t, s in self.stt.get(self.current_state, [])]
            next_state = next_state[0]
            # make the call, or return the state or whatever
            self.current_state = next_state
        raise Error('Transition %s from %s not allowed' % (transition, self.current_state))



#!/usr/bin/env python

# emotiv.py
# Copyright (C) 2013  Bastian Venthur
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


from Crypto.Cipher import AES
import numpy as np

import usb.core
import usb.util

from libmushu.amplifier import Amplifier


VENDOR_ID = 0x1234
PRODUCT_ID = 0xed02

ENDPOINT_IN = usb.util.ENDPOINT_IN | 2  # second endpoint


class Epoc(Amplifier):

    def __init__(self):
        # find amplifier
        self.dev = usb.core.find(idVendor=VENDOR_ID, idProduct=PRODUCT_ID)
        if self.dev is None:
            raise RuntimeError('Emotiv device is not connected.')
        # pyusb docs say you *have* to call set_configuration, but it does not
        # work unless i *don't* call it.
        #dev.set_configuration()
        # get the serial number
        serial = usb.util.get_string(self.dev, 17, self.dev.iSerialNumber)
        # claim the device and it's two interfaces
        if self.dev.is_kernel_driver_active(0):
            self.dev.detach_kernel_driver(0)
        usb.util.claim_interface(self.dev, 0)
        if self.dev.is_kernel_driver_active(1):
            self.dev.detach_kernel_driver(1)
        usb.util.claim_interface(self.dev, 1)
        # prepare AES
        self.cipher = AES.new(self.generate_key(serial, True))
        # internal states for battery and impedance we have to store since it
        # is not sent with every frame.
        self._battery = 0
        self._quality = [0 for i in range(14)]
        # channel info
        self.channel = ['Counter', 'Battery',
                        'F3', 'FC5', 'AF3', 'F7', 'T7', 'P7', 'O1',
                        'O2', 'P8', 'T8', 'F8', 'AF4', 'FC6', 'F4',
                        'Gyro 1', 'Cyro 2',
                        'Quality F3', 'Quality FC5', 'Quality AF3',
                        'Quality F7', 'Quality T7', 'Quality P7',
                        'Quality 01', 'Quality O2', 'Quality P8',
                        'Quality T8', 'Quality F8', 'Quality AF4',
                        'Quality FC6', 'Quality F4']

    def get_data(self):
        try:
            raw = self.dev.read(ENDPOINT_IN, 32, 1, timeout=1000)
            raw = self.decrypt(raw)
            data = self.parse_raw(raw)
            data = np.array(data)
        except Exception as e:
            print e
            data = np.array()
        return data.reshape(1, -1), []

    @staticmethod
    def is_available():
        if usb.core.find(idVendor=VENDOR_ID, idProduct=PRODUCT_ID) is None:
            return False
        else:
            return True

    def generate_key(self, sn, research=True):
        """Generate the encryption key.

        The key is based on the serial number of the device and the information
        weather it is a research- or consumer device.

        """
        if research:
            key = ''.join([sn[15], '\x00', sn[14], '\x54',
                           sn[13], '\x10', sn[12], '\x42',
                           sn[15], '\x00', sn[14], '\x48',
                           sn[13], '\x00', sn[12], '\x50'])
        else:
            key = ''.join([sn[15], '\x00', sn[14], '\x48',
                           sn[13], '\x00', sn[12], '\x54',
                           sn[15], '\x10', sn[14], '\x42',
                           sn[13], '\x00', sn[12], '\x50'])
        return key

    def decrypt(self, raw):
        """Decrypt a raw package."""
        data = self.cipher.decrypt(raw[:16]) + self.cipher.decrypt(raw[16:])
        tmp = 0
        for i in range(32):
            tmp = tmp << 8
            tmp += ord(data[i])
        return tmp

    def parse_raw(self, raw):
        """Parse raw data."""
        data = []
        shift = 256
        # 1x counter / battery (8 bit)
        # if the first bit is not set, the remaining 7 bits are the counter
        # otherwise the remaining bits are the battery
        shift -= 8
        tmp = (raw >> shift) & 0b11111111
        if tmp & 0b10000000:
            # battery
            counter = 128
            self._battery = tmp & 0b01111111
        else:
            # counter
            counter = tmp & 0b01111111
        data.append(counter)
        data.append(self._battery)
        # 7x data (14 bit)
        for i in range(7):
            shift -= 14
            data.append((raw >> shift) & 0b11111111111111)
        # 1x quality (14 bit)
        # we assume it is the contact quality for an electrode, the counter
        # gives the number of the electrode. since we only have 14 electrodes
        # we only take the values from counters 0..13 and 64..77
        shift -= 14
        tmp = (raw >> shift) & 0b11111111111111
        if counter < 128:
            if counter % 64 < 14:
                self._quality[counter % 64] = tmp
        # 1x ??? (14 bit)
        shift -= 14
        # 7x data (14 bit)
        for i in range(7):
            shift -= 14
            data.append((raw >> shift) & 0b11111111111111)
        # 2x gyroscope (8 bit)
        # the first bit denotes the sign the remaining 7 bits the number
        for i in range(2):
            shift -= 8
            tmp = (raw >> shift) & 0b01111111
            tmp -= 100
            if (raw >> shift) & 0b10000000:
                tmp *= -1
            data.append(tmp)
        # 1x ??? (8 bit)
        shift -= 8
        data.extend(self._quality)
        return [int(i) for i in data]


if __name__ == '__main__':
    amp = Epoc()
    print 'Reading...'
    while 1:
        try:
            print amp.get_data()
        except Exception as e:
            print e
            break



#!/usr/bin/env python


from distutils.core import setup

import libmushu

setup(
    name = 'Mushu',
    version = libmushu.__version__,
    description = 'Brain Computer Interfacing signal acquisition software.',
    long_description = 'A Python library for BCI signal acquisition. It can be used as a stand-alone application or as a library.',
    author = 'Bastian Venthur',
    author_email = 'bastian.venthur@tu-berlin.de',
    url = 'http://venthur.github.io/mushu/',
    download_url = 'http://github.com/venthur/mushu/',
    license = 'GPL2',
    platforms = 'any',
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Healthcare Industry',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Education',
        'Topic :: Scientific/Engineering :: Human Machine Interfaces',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',
        'Topic :: Software Development :: Libraries',
        ],
    packages = ['libmushu', 'libmushu.driver'],
    scripts = ['mushu.py'],
)




# randomamp.py
# Copyright (C) 2013  Bastian Venthur
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from __future__ import division

import time
import math
import json

import numpy as np

from libmushu.amplifier import Amplifier


PRESETS = [['Sine wave at 50Hz, 16Channels', {'f' : 1, 'fs' : 10, 'channels' : 16}],
           ['Sine wave at 1kHz, 128Channels', {'f' : 3, 'fs' : 100, 'channels' : 128}]
          ]


class SinusAmp(Amplifier):
    """An amplifier that produces sinus data."""

    def __init__(self):
        self.presets = PRESETS
        self.configure(**self.presets[0][1])
        self.last_sample = time.time()

    @property
    def sample_len(self):
        return 1 / self.fs

    @property
    def elapsed(self):
        return time.time() - self.last_sample

    def get_data(self):
        # simulate blocking until we have enough data
        elapsed = self.elapsed
        if elapsed < self.sample_len:
            time.sleep(self.sample_len - elapsed)
        dt = self.elapsed
        samples = math.floor(self.fs * dt)
        t = np.linspace(self.last_sample, self.last_sample + dt, samples)
        t = np.array([t for i in range(self.channels)]).T
        data = np.sin(np.pi*2*t*self.f)
        self.last_sample = time.time()
        return data, []

    def configure(self, f, fs, channels):
        self.f = f
        self.fs = fs
        self.channels = channels

    def get_channels(self):
        return ['Ch_%d' % i for i in range(self.channels)]

    def get_sampling_frequency(self):
        return self.fs

    @staticmethod
    def is_available():
        return True


if __name__ == '__main__':
    pass
    amp = SinusAmp()
    amp.configure(channels=2, fs=10, f=2)
    amp.start()
    for i in range(20):
        t = time.time()
        print amp.get_data()
    amp.stop()
#
#    # the same using a context manager, start and stop are called by the
#    # context manager
#    with amp as a:
#        for i in range(10):
#            print a.get_data()
